/*
*            k145u.h
*       заголовочный файл
*  базового модуля имиации часов 
*   на основе К145ИК1901
*  расширенная версия на основе
*  ардуино-нано (про-мини),
*  RTC DS3231,
*  10-позиционного ЖКИ дисплея на NJU6432x
*  и минимальной клавиатуры (4-5 кнопок)
*  используюшей курсор с привязкой
*  к "пиктограммам"
*  с возможностью расширения функционала ПРВ
*  
*/
//-------------------------------------------------------
 #ifndef _k145u_H  // Блокируем повторное включение этого модуля
 #define _k145u_H

//------------------
// подключение функционала референса arduinoIDE
#if ARDUINO >= 100
 #include "Arduino.h"
#else
 #include "WProgram.h"
#endif

//------------------

//---------------------------------------------------------
// #include <avr/pgmspace.h> // подключить библиотеку работ с таблицами в ПЗУ
// #include <EEPROM.h> // "стандартная" ардуино библиотека ЕЕПРОМ
// #include "sksi.h" // заголовочник сканера кноп консоли

//         ------- devices -------

#include "mainrtc.h" // заголовочник устройства "главные часы" (RTC)
#include "secmetr.h" // заголовочник субустройства "секундомер"
#include "timer.h" // заголовочный субустройства
                   //  "таймер обратного отсчета"
#include "termo.h" // заголовочный *.h файл датчика DS18B20

//------------------------------------------------------------------------
// раскладка управляющих линий (выводов)

//------------------------------------------------------------------------
// определение/инициализация входных линий МК в соответствии
// с номерами кнопок (сенсоров),
// активный уровень на линии сенсора = 0
const byte PinToV =18; // D18 линия кнопки S4(Vkey)
const byte PinToM =17; // D17 линия кнопки S3(Down)
const byte PinToP =16; // D16 линия кнопки S2(Up)
const byte PinToR =15; // D15 линия кнопки S1(Right)
const byte PinToL =14; // D14 линия кнопки S0(Left)

// обьявление и инициализация управляющих выводов NJU6432
const byte PinNoInh =9; // линия управления дисплея (бланкирование)
const byte PinDn =10; // линия данных дисплея
const byte PinScl =11; // тактовый строб сопровождения данных дисплея
const byte PinCe =12; // линия разрешения обмена дисплея

// константа интервала опросов клавиатуры (сенсоров) в милисекундах
const byte skstm = 35;

//------------------------------------------------------------------------
// инициализация управляющих выводов RTC
const byte L_QS = 3; // вывод строба/прерываний RTC
const byte L_SDA = 4; // вывод данных I2C
const byte L_SCL = 5; // вывод строба I2C

//------------------------------------------------------------------------
// инициализация системных силовых выводов аппаратной обвязки 
const byte HLtmr = 8; // сигнального выхода таймера
const byte HLbud = 2; // сигнальный вывод комплекта бодунов *!!
const byte DTM = 7; // сигнальный вывод звуковой головки

// инициализация линии DS18B20
const byte DQ = 6;

// инициализация констант схемотехнических активных уровней для
// системных силовых выводов аппаратной обвязки
const byte HLtmrWrk = 1; // сигнального выхода таймера
const byte HLbudWrk = 1; // сигнальный вывод комплекта бодунов *!!

//------------------------------------------------------------------------
// система контроля питающего напряжения

const byte Apwr = 7; // вход АЦП
/*
 * ******************************************
 * обязательно учитывать, что в ардуино-нано 
 * (или про-мини) могут быть "по умолчанию"
 * выставлены BOD - фузы !!!
 * к примеру:
 * BODLEVEL2:BODLEVEL1:BODLEVEL0 = 1:0:1
 * т.е. аппаратный сброс 
 * при уровне питающего напряжения
 * 2,5 - 2,7 - 2,9 вольта
 * ******************************************
 */
// индикация необходимости включить зарядку
void lowBat();

//------------------------------------------------------------------------
// указатели на функции возврата в предыдущий модуль устройства
// (стек возврата из субменю) 3 уровня
// "растет вверх"
// заполняется при вызове субменю,
// опустошается по завершении выполнения
// функции возврата из субменю
// extern volatile KeyPtr RetSteckUH; // уровень 4
// extern volatile KeyPtr RetSteckUL; // уровень 3
volatile KeyPtr RetSteckU = stub; // уровень 2
volatile KeyPtr RetSteckH = stub; // уровень 1
volatile KeyPtr RetSteckL = stub; // уровень 0
// указатель на выполняемую при "все отключены"
// функцию возврата на предыдущий уровень
volatile KeyPtr RetPtr = stub;

// указатели на функции инициализации
// имеющихся в системе "устройств"
volatile KeyPtr dev0_init = DevClockInit; // основная панель главных часов
volatile KeyPtr dev1_init = secmetrInit; // секундомер
volatile KeyPtr dev2_init = Tmr_Init; // таймер обратного отсчета
volatile KeyPtr dev3_init = AdjRtcInit; // настройка параметров RTC и будильников
volatile KeyPtr dev4_init = TermoInit; // термодатчик DS18b20
/*
volatile KeyPtr dev5_init = stub; //
volatile KeyPtr dev6_init = stub; //
volatile KeyPtr dev7_init = stub; //
*/

//------------------------------------------------------------------------

#endif

//-------------конец файла/end of file---------------------
